class Content {
	private width : number = 19;
	private height : number = 34;
	private size : number = 324;

	private x : number;
	private y : number;
	private i : number;
	private val : number;
	
	private displayCanvas : HTMLCanvasElement;
	private videoCanvas : HTMLCanvasElement;

	private displayCtx : CanvasRenderingContext2D;
	private videoCtx : CanvasRenderingContext2D;

	private video : HTMLVideoElement;

	private palette16 : number[][] = [
		[255, 0, 0], 
		[0, 255, 0], 
		[0, 0, 255],  
		[255, 255, 0], 
		[255, 0, 255],    
		[0, 255, 255],  
		[255, 128, 0], 
		[128, 0, 255], 
		[0, 255, 128],   
		[255, 255, 255],  
		[0, 0, 0],       
		[128, 128, 128], 
		[128, 64, 0],
		[0, 128, 128],  
		[255, 192, 203],
		[64, 128, 0]  
	];

	private palettegray : number[][] = [
	    [0, 0, 0],
	    [17, 17, 17],
	    [34, 34, 34],
	    [51, 51, 51],
	    [68, 68, 68],
	    [85, 85, 85],
	    [102, 102, 102],
	    [119, 119, 119],
	    [136, 136, 136],
	    [153, 153, 153],
	    [170, 170, 170],
	    [187, 187, 187],
	    [204, 204, 204],
	    [221, 221, 221],
	    [238, 238, 238],
	    [255, 255, 255] 
	];

	private palette : number[][][] = [];
	private gray : number[][][] = [];

	constructor () {
		const dpr  : number = window.devicePixelRatio;

		this.displayCanvas  = document.getElementById('display') as HTMLCanvasElement;
		this.displayCtx = this.displayCanvas.getContext('2d');

		this.displayCanvas.height = this.height;
		this.displayCanvas.width = this.width;
		//document.getElementById('video') as HTMLCanvasElement; 
		this.videoCanvas  = document.createElement('canvas') as HTMLCanvasElement;
		this.videoCtx = this.videoCanvas.getContext('2d');

		for (let a = 0; a < this.palette16.length; a++) {
			for (let b = 0; b < this.palette16.length; b++) {
				this.palette.push([this.palette16[a], this.palette16[b]]);
			}
		}

		for (let a = 0; a < this.palettegray.length; a++) {
			for (let b = 0; b < this.palettegray.length; b++) {
				this.gray.push([this.palettegray[a], this.palettegray[b]]);
			}
		}
		
		this.startCamera();
	}

	async startCamera () {
		const constraints : MediaStreamConstraints = {
			video : {
				width: { ideal: 640 },
				height: { ideal: 480 },
				facingMode: 'environment'
			},
			audio: false
		}
		try {
			const stream = await navigator.mediaDevices.getUserMedia(constraints);

			this.video = document.createElement('video') as HTMLVideoElement;

			this.video.srcObject = stream;
			this.video.height = stream.getVideoTracks()[0].getSettings().height;
			this.video.width  = stream.getVideoTracks()[0].getSettings().width;
			this.video.play();

			this.videoCanvas.height = this.video.height;
			this.videoCanvas.width  = this.video.width;

			(document.querySelector('#data') as HTMLDivElement).innerText = `${this.video.width}x${this.video.height}`;

			this.video.addEventListener('play', this.drawCanvas.bind(this));
		} catch (error) {
			console.error('Error accessing camera:', error);
		}
	}

	private drawCanvas () {
		this.videoCtx.drawImage(this.video, 0, 0, this.videoCanvas.width, this.videoCanvas.height);
		this.parse();
		requestAnimationFrame(this.drawCanvas.bind(this));
	}

	private parse () {
		const rgba : ImageData = this.videoCtx.getImageData( 
			0, 0, this.videoCanvas.width, this.videoCanvas.height 
		).data as any;
		
  		let code : any = null

  		try {
  			//@ts-ignore
  			code = jsQR(rgba, this.videoCanvas.width, this.videoCanvas.height);
  		} catch (err) {
  			//
  		}

  		if (code !== null && code.binaryData.length === this.size) {
    		//(document.querySelector('#data') as HTMLDivElement).innerText = code.binaryData.toString();
    		//console.dir(code);
    		this.display(code.binaryData);
  		} else {
  			//console.log('No QR')
  		}
	}

	display (data : any[]) {
		let pos : number = 0;
		const palette : number[][][] = data[0] === 0 ? this.gray : this.palette; 
		for (this.i = 1; this.i < data.length; this.i++) {
			this.val = data[this.i];
			for (let color of palette[this.val]) {
				this.x = pos % this.width;
				this.y = Math.floor(pos / this.width);
				this.displayCtx.fillStyle = `rgba(${color[0]},${color[1]},${color[2]},1.0)`;
				this.displayCtx.fillRect(this.x, this.y, 1, 1);
				pos += 1
			}
		}
	}
}

function start () {
	new Content();
};